--
-- This is the initialization function. It gets called every time Rainlendar
-- is started or refreshed. The initialization function must be defined in the
-- skin file.
--
function Shadow4_Countdown_Initialize(skin, window)
  -- Create a timer which updates the data once per second
  Rainlendar_CreateTimer(1000, Shadow4_Countdown_OnTimer, skin .. "|" .. window)

    -- Change the text in the window
    event = Rainlendar_GetVariable(skin, window, "Countdown_event")
    Rainlendar_SetItemValue(window, "Countdown.text", "text", event)
  
  -- Run the timer callback so the data is updated immediately
  Shadow4_Countdown_OnTimer(skin .. "|" .. window)
end

--
-- A callback function for the timer. This gets called every time the timer triggers.
-- Returns true so the timer will go on forever (false would stop the timer).
--
function Shadow4_Countdown_OnTimer(userData)
  -- Split the user data
  _, _, skin, window = userData:find("(.*)|(.*)")

    time = Rainlendar_GetVariable(skin, window, "Countdown_time")
    date = Rainlendar_GetVariable(skin, window, "Countdown_date")

  _, _, tHour, tMin, tSec = time:find("(%d-):(%d+):(%d+)")
  _, _, tYear, tMonth, tDay = date:find("(%d+)-(%d+)-(%d+)")

  -- Rainlendar_Log(date .. " " .. tYear .. tMonth .. tDay);

  time = ""
  if (tHour and tMin and tSec and tYear and tMonth and tDay) then
    target = os.time{year=tYear, month=tMonth, day=tDay, hour=tHour, min=tMin, sec=tSec}
    current = os.time()
  
    diff = math.abs(target - current)
  
    days = math.floor(diff / (60 * 60 * 24))
    hours = math.floor((diff % (60 * 60 * 24)) / (60 * 60))
    mins = math.floor((diff % (60 * 60)) / 60)
    secs = math.floor(diff % 60)
    
    time = string.format("%03i:%02i:%02i:%02i", days, hours, mins, secs)
  else
    time = "Error"
  end
  
    -- Calculate the difference from now to the target date
    Rainlendar_SetItemValue(window, "Countdown.time", "text", time)
    Rainlendar_Redraw(0, window)

  return True
end
