function Shadow4_Moonphase_Initialize(skin, window)
-- Create a timer which updates once per hour
  Rainlendar_CreateTimer(60 * 60 * 1000, Shadow4_Moonphase_OnTimer, skin .. "|" .. window)

  if Rainlendar_GetVariable(skin, window, "Southern_hemisphere") == "1" then
    Rainlendar_SetItemValue(window, "Moon Phase.image", "element", "bitmap.moonphase.phases.southern")
  end

  -- Run the timer callback so the window is updated immediately
  Shadow4_Moonphase_OnTimer(skin .. "|" .. window)
end

function Shadow4_Moonphase_OnTimer(userData)
  -- Split the user data
  _, _, skin, window = userData:find("(.*)|(.*)")
  
  phase = Shadow4_CalculateMoonPhase(os.date("%Y"), os.date("%m"), os.date("%d"))

  Rainlendar_Log("Moonphase: " .. tostring(phase));

  -- Map the value to the frame
  frame = phase * 16

  Rainlendar_Log("Moonphase frame: " .. tostring(frame));
  
  -- Frames start from 1 and not from 0 so add 1 to the phase
  frame = math.floor(frame) + 1

  Rainlendar_Log("Moonphase frame: " .. tostring(frame));

  Rainlendar_SetItemValue(window, "Moon Phase.image", "frame", tostring(frame))
  
  phase = ""
  if frame == 1 then
    phase = Rainlendar_GetString("New moon")
  elseif frame == 2 or frame == 3 or frame == 4 then
    phase = Rainlendar_GetString("Waxing crescent")
  elseif frame == 5 then
    phase = Rainlendar_GetString("First quarter")
  elseif frame == 6 or frame == 7 or frame == 8 then
    phase = Rainlendar_GetString("Waxing gibbous")
  elseif frame == 9 then
    phase = Rainlendar_GetString("Full moon")
  elseif frame == 10 or frame == 11 or frame == 12 then
    phase = Rainlendar_GetString("Waning gibbous")
  elseif frame == 13 then
    phase = Rainlendar_GetString("Last quarter")
  else
    phase = Rainlendar_GetString("Waning crescent")
  end
  
  Rainlendar_SetItemValue(window, "Moon Phase.text", "text", phase)
  
  Rainlendar_Redraw(0, window)
end

-- Algorithm from http://www.voidware.com/moon_phase.htm
function Shadow4_CalculateMoonPhase(y, m, d)
  y = tonumber(y)
  m = tonumber(m)
  d = tonumber(d)

  if (m < 3) then
    y = y - 1
    m = m + 12
  end
  
  m = m + 1
  c = 365.25 * y
  e = 30.6 * m
  jd = c + e + d - 694039.09    -- jd is total days elapsed  
  jd = jd / 29.53               -- divide by the moon cycle (29.53 days)
  jd = jd - math.floor(jd)      -- subtract integer part to leave fractional part of original jd
  return jd
end
