--
-- This is the initialization function. It gets called every time Rainlendar
-- is started or refreshed. The initialization function must be defined in the
-- skin file.
--
function Shadow4_MSWeather_Initialize(skin, window)
  -- Create a timer which updates the data once per hour
  Rainlendar_CreateTimer(60 * 60 * 1000, Shadow4_MSWeather_OnTimer, skin .. "|" .. window)

  -- Set global event for power resume
  Rainlendar_SetEventHandler(Shadow4_MSWeather_OnPowerResume, 1000, "", skin .. "|" .. window)
  
  -- Run the timer callback so the data is updated immediately
  Shadow4_MSWeather_OnTimer(skin .. "|" .. window)
end

--
-- A callback function for the timer. This gets called every time the timer triggers.
-- Returns true so the timer will go on forever (false would stop the timer).
--
function Shadow4_MSWeather_OnTimer(userData)
  -- Split the user data
  _, _, skin, window = userData:find("(.*)|(.*)")

  location = Rainlendar_GetVariable(skin, window, "Weather_location")
  celsius = Rainlendar_GetVariable(skin, window, "Weather_celsius")

  if location then
    url = "http://weather.service.msn.com/data.aspx?culture=en-US&src=outlook&weasearchstr=" .. Shadow4_MSWeather_EncodeURI(location)
    if celsius and celsius == "1" then
        url = url .. "&weadegreetype=C" 
    end
    Rainlendar_Log("Downloading: " .. url)
    Rainlendar_Download(url, Shadow4_MSWeather_OnDownload, userData)
    return True
  else
    Rainlendar_Log("The location variable is not defined.")
    return False
  end
end

-- A callback function which gets called when computer wakes up from sleep.
-- This reloads the weather data.
function Shadow4_MSWeather_OnPowerResume(eventData, userData)
  Shadow4_MSWeather_OnTimer(userData)
end

--
-- A callback function for the downloading. This gets called after the given file
-- has been downloaded from the net. The data is the downloaded file.
-- In this case the weather rss feed is parsed and the view is filled
-- in with the data.
--
function Shadow4_MSWeather_OnDownload(result, data, userData)
  -- Split the user data
  _, _, skin, window = userData:find("(.*)|(.*)")

  iconMap = {}
  iconMap["0"] = "windy" -- tornado
  iconMap["1"] = "windy" -- tropical storm
  iconMap["2"] = "windy" -- hurricane
  iconMap["3"] = "thunderstorm" -- severe thunderstorms
  iconMap["4"] = "thunderstorm" -- thunderstorms
  iconMap["5"] = "rain_snow" -- mixed rain and snow
  iconMap["6"] = "rain_snow" -- mixed rain and sleet
  iconMap["7"] = "snow" -- mixed snow and sleet
  iconMap["8"] = "rain_snow" -- freezing drizzle
  iconMap["9"] = "rain_light" -- drizzle
  iconMap["10"] = "rain_snow" -- freezing rain
  iconMap["11"] = "rain_light" -- showers
  iconMap["12"] = "rain_light" -- showers
  iconMap["13"] = "snow_light" -- snow flurries
  iconMap["14"] = "snow_light" -- light snow showers
  iconMap["15"] = "snow_light" -- blowing snow
  iconMap["16"] = "snow" -- snow
  iconMap["17"] = "snow" -- hail
  iconMap["18"] = "sleet" -- sleet
  iconMap["19"] = "fog" -- dust
  iconMap["20"] = "fog" -- foggy
  iconMap["21"] = "fog" -- haze
  iconMap["22"] = "fog" -- smoky
  iconMap["23"] = "fog" -- blustery
  iconMap["24"] = "windy" -- windy
  iconMap["25"] = "clear_day" -- cold
  iconMap["26"] = "cloudy" -- cloudy
  iconMap["27"] = "cloudy_mostly_night" -- mostly cloudy (night)
  iconMap["28"] = "cloudy_mostly_day" -- mostly cloudy (day)
  iconMap["29"] = "cloudy_partly_night" -- partly cloudy (night)
  iconMap["30"] = "cloudy_partly_day" -- partly cloudy (day)
  iconMap["31"] = "clear_night" -- clear (night)
  iconMap["32"] = "clear_day" -- sunny
  iconMap["33"] = "cloudy_partly_night" -- fair (night)
  iconMap["34"] = "cloudy_partly_day" -- fair (day)
  iconMap["35"] = "rain_snow" -- mixed rain and hail
  iconMap["36"] = "clear_day" -- hot
  iconMap["37"] = "thunderstorm" -- isolated thunderstorms
  iconMap["38"] = "thunderstorm" -- scattered thunderstorms
  iconMap["39"] = "thunderstorm" -- scattered thunderstorms
  iconMap["40"] = "rain_light" -- scattered showers
  iconMap["41"] = "snow" -- heavy snow
  iconMap["42"] = "snow_light" -- scattered snow showers
  iconMap["43"] = "snow" -- heavy snow
  iconMap["44"] = "cloudy_partly_day" -- partly cloudy
  iconMap["45"] = "thunderstorm" -- thundershowers
  iconMap["46"] = "snow_light" -- snow showers
  iconMap["47"] = "thunderstorm" -- isolated thundershowers
  iconMap["3200"] = "na" -- not available

  if (result == 200) then
    -- <weatherdata xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
    -- <weather weatherlocationcode="fr:FI_Tampere" weatherlocationname="Tampere, Finland" url="http://a.msn.com/54/en-US/ct61.498,23.772?ctsrc=outlook" imagerelativeurl="http://blob.weather.microsoft.com/static/weather4/en-us/" degreetype="C" provider="Foreca" attribution="http://www.foreca.com/" attribution2="Foreca" lat="61.498" long="23.772" timezone="3" alert="" entityid="32669" encodedlocationname="Tampere%2C+Finland">
    -- <current temperature="7" skycode="26" skytext="Cloudy" date="2019-04-20" observationtime="12:00:00" observationpoint="Tampere, Finland" feelslike="7" humidity="70" winddisplay="2 km/h West" day="Saturday" shortday="Sat" windspeed="2 km/h"/>
    -- <forecast low="0" high="13" skycodeday="27" skytextday="Cloudy" date="2019-04-19" day="Friday" shortday="Fri" precip=""/>
    -- <forecast low="1" high="12" skycodeday="26" skytextday="Cloudy" date="2019-04-20" day="Saturday" shortday="Sat" precip="0"/>
    -- <forecast low="4" high="13" skycodeday="30" skytextday="Partly Sunny" date="2019-04-21" day="Sunday" shortday="Sun" precip="0"/>
    -- <forecast low="4" high="14" skycodeday="28" skytextday="Mostly Cloudy" date="2019-04-22" day="Monday" shortday="Mon" precip="0"/>
    -- <forecast low="3" high="15" skycodeday="30" skytextday="Partly Sunny" date="2019-04-23" day="Tuesday" shortday="Tue" precip="0"/>
    -- <toolbar timewindow="60" minversion="1.0.1965.0"/>
    -- </weather>
    -- </weatherdata>

    -- Create a link from the title
    _, _, title = data:find("weatherlocationname=\"(.-)\"")
    _, _, link = data:find("url=\"(.-)\"")

    if title and link then
      title = "[[" .. link .. "][" .. title .. "]]" 
      
      -- Put the title to the window
      Rainlendar_SetItemValue(window, "Weather.title", "text", title)

      -- Current conditions
      _, _, degrees, code, conditions, date, time, humidity, wind = data:find("<current temperature=\"(.-)\" skycode=\"(.-)\" skytext=\"(.-)\" date=\"(.-)\" observationtime=\"(.-)\" observationpoint.- humidity=\"(.-)\" winddisplay=\"(.-)\" day.-/>")

      if degrees and conditions and date and time and humidity and wind then
        -- Set the icon
        Rainlendar_SetItemValue(window, "Weather.icon.large", "element", "bitmap.icon.weather." .. iconMap[code])

        temperatureUnit = "F"
        celsius = Rainlendar_GetVariable(skin, window, "Weather_celsius")
        if celsius and celsius == "1" then
          temperatureUnit = "C"
        end
  
        -- Set the conditions
        Rainlendar_SetItemValue(window, "Weather.degrees", "text", degrees .. "°" .. temperatureUnit)

        text = ""
        text = text .. date .. " " .. time .. "\n"
        text = text .. conditions .. " " .. degrees .. "°" .. temperatureUnit .. "\n"
        text = text .. Rainlendar_GetString("Wind:") .. " " .. wind .. "\n"
        text = text .. Rainlendar_GetString("Humidity:") .. " " .. humidity .. "\n"
        text = text .. "\n\n" .. Rainlendar_GetString("Provider:") .. " [[" .. url .. "][MS Weather]]"
    
        Rainlendar_SetItemValue(window, "Weather.icon.large", "tooltipHeader", title)
        Rainlendar_SetItemValue(window, "Weather.icon.large", "tooltipText", text)

        -- Loop throught all <forecast>:s.
        count = 1
        for forecast in string.gmatch(data, "<forecast(.-)/>") do

          -- <forecast low="0" high="13" skycodeday="27" skytextday="Cloudy" date="2019-04-19" day="Friday" shortday="Fri" precip=""/>
          _, _, forecastDate = forecast:find("date=\"(.-)\"")
          if Shadow4_MSWeather_CompareDates(date, forecastDate) then
            _, _, day = forecast:find("shortday=\"(.-)\"")
            _, _, low = forecast:find("low=\"(.-)\"")
            _, _, high = forecast:find("high=\"(.-)\"")
            _, _, code = forecast:find("skycodeday=\"(.-)\"")

            if day and low and high and code then
              Rainlendar_SetItemValue(window, "Weather.day." .. count, "text", day)
              Rainlendar_SetItemValue(window, "Weather.icon.small." .. count, "element", "bitmap.icon.weather." .. iconMap[code])
              Rainlendar_SetItemValue(window, "Weather.degrees.low." .. count, "text", Rainlendar_GetString("Min:") .. " " .. low .. "°" .. temperatureUnit)
              Rainlendar_SetItemValue(window, "Weather.degrees.high." .. count, "text", Rainlendar_GetString("Max:") .. " " .. high .. "°" .. temperatureUnit)
            end
            
            count = count + 1

            if count >= 3 then
              break
            end
          end
        end
        
      end
    end
  else
    -- Download failed
    Rainlendar_SetItemValue(window, "Weather.title", "text", Rainlendar_GetString("Download failed!"))
  end
  
  Rainlendar_Redraw(0, window)
end

function Shadow4_MSWeather_EncodeURI(str)
  if (str) then
    str = string.gsub (str, "\n", "\r\n")
    str = string.gsub (str, "([^%w ])",
      function (c) return string.format ("%%%02X", string.byte(c)) end)
    str = string.gsub (str, " ", "+")
   end
   return str
end

function Shadow4_MSWeather_CompareDates(current, forecast)
  if current and forecast then
    return forecast:gsub('-', '') > current:gsub('-', '')
  end
  return false
end
