-- This is a table which will contain the scanned folder contents
g_Shadow4_Photos_Cache = {}

--
-- This is the initialization function. It gets called every time Rainlendar
-- is started or refreshed. The initialization function must be defined in the
-- skin file.
--
function Shadow4_Photos_Initialize(skin, window)
  math.randomseed(os.time())

  -- Create a timer which updates the images
  time = Rainlendar_GetVariable(skin, window, "Photos_update")
  if (time) then
    time = tonumber(time) * 60 * 1000   -- timer uses ms so we need to convert the mins to milliseconds
  end
  if (time <= 0) then
    time = 60 * 60 * 1000
  end
  Rainlendar_CreateTimer(time, Shadow4_Photos_OnTimer, skin .. "|" .. window)

  -- Set handlers for mouse enter and leave events
  Rainlendar_SetEventHandler(Shadow4_Photos_OnMouseEnter, 7, window)
  Rainlendar_SetEventHandler(Shadow4_Photos_OnMouseLeave, 8, window)

  -- Set global event for power resume
  Rainlendar_SetEventHandler(Shadow4_Photos_OnPowerResume, 1000, "", skin .. "|" .. window)

  -- Check if the folder is defined and read the files from it
  Rainlendar_Log("Created cache for " ..  window)
  g_Shadow4_Photos_Cache[window] = {}
  folder = Rainlendar_GetVariable(skin, window, "Photos_folder")
  if folder ~= "" then
    for filename in dirtree(folder) do
      if string.find(filename:lower(), ".jpg") or 
         string.find(filename:lower(), ".png") or 
         string.find(filename:lower(), ".bmp") or 
         string.find(filename:lower(), ".gif") then
        filename = string.gsub(filename, "\\", "/")
        table.insert(g_Shadow4_Photos_Cache[window], filename) 
      end
    end
  end

  -- Set the element name to unique for the windows so multiple windows can have different images
  Rainlendar_SetItemValue(window, "Photos.photo", "element", "bitmap.photos.photo." .. window)
  
  -- Run the timer callback so the data is updated immediately
  Shadow4_Photos_OnTimer(skin .. "|" .. window)
end

--
-- A callback function for the timer. This gets called every time the timer triggers.
-- Returns true so the timer will go on forever (false would stop the timer).
--
function Shadow4_Photos_OnTimer(userData)
  -- Split the user data
  _, _, skin, window = userData:find("(.*)|(.*)")

  page = 1
  bRnd = Rainlendar_GetVariable(skin, window, "Photos_random")
  if (bRnd ~= "0") then
    if #g_Shadow4_Photos_Cache[window] > 0 then
      page = math.random(#g_Shadow4_Photos_Cache[window])
    else
      page = math.random(500)
    end
  end
  
  Shadow4_Photos_Download(page, userData)
  return true
end

-- A callback function which gets called when computer wakes up from sleep.
-- This loads the image.
function Shadow4_Photos_OnPowerResume(eventData, userData)
  Shadow4_Photos_OnTimer(userData)
end

-- Function which starts the downloading
function Shadow4_Photos_Download(page, userData)
  -- Split the user data
  _, _, skin, window = userData:find("(.*)|(.*)")

  Rainlendar_SetItemValue(window, "Photos.infotext", "text", Rainlendar_GetString("Loading..."))
  Rainlendar_SetItemValue(window, "Photos.infotext", "visible", "1")
  Rainlendar_Redraw(0, window)

  Rainlendar_Log("Count: " .. #g_Shadow4_Photos_Cache[window] .. " page: " .. page)

  if (#g_Shadow4_Photos_Cache[window] == 0) then
    -- If you use this as an example for you skin please change the Flickr API key (http://www.flickr.com/services/api/keys/).
    apiKey = "4ee7dc4c2943311b2f14d5f692cbee6a"
    url = "https://api.flickr.com/services/rest/?method=flickr.interestingness.getList&api_key=" .. apiKey .. "&extras=owner_name&per_page=1&page=" .. page
    Rainlendar_Log("Downloading: " .. url)
    Rainlendar_Download(url, Shadow4_Photos_OnDownload, userData)
  else
    -- Read the image from local folder
    if (#g_Shadow4_Photos_Cache[window] >= page) then
      url = "file://" .. g_Shadow4_Photos_Cache[window][page]
      Rainlendar_Log("Downloading: " .. url)
      Rainlendar_DownloadElement(url, Shadow4_Photos_OnDownloadElement, "bitmap.photos.photo." .. window, skin, window)

      _, _, name = url:find(".*/(.*)")
      Rainlendar_SetItemValue(window, "Photos.notes", "text", name)
      Rainlendar_SetItemValue(window, "Photos.index", "text", page .. "/" .. #g_Shadow4_Photos_Cache[window])

      -- Change the action in the show-button so it'll go to correct page
      Rainlendar_SetItemValue(window, "Photos.button.show", "action", "Shadow4_Photos_Execute('" .. url .. "')")
    else
      Rainlendar_SetItemValue(window, "Photos.infotext", "text", Rainlendar_GetString("Download failed!"))
      Rainlendar_Redraw(0, window)
    end
  end
end

--
-- A callback function for the downloading. This gets called after the given file
-- has been downloaded from the net.
--
function Shadow4_Photos_OnDownload(result, data, userData)

  -- Split the user data
  _, _, skin, window = userData:find("(.*)|(.*)")

  if (result == 200) then
    -- Parse the information from the returned xml
    _, _, id = data:find(' id="(%d-)" ')
    _, _, owner = data:find(' owner="(.-)" ')
    _, _, ownername = data:find(' ownername="(.-)" ')
    _, _, secret = data:find(' secret="(%x-)" ')
    _, _, server = data:find(' server="(%d-)" ')
    _, _, farm = data:find(' farm="(%d-)" ')
    _, _, title = data:find(' title="(.-)" ')
    _, _, page = data:find(' page="(%d-)" ')
    _, _, pages = data:find(' pages="(%d-)" ')

    if (id and farm and server and secret) then
      url = "http://farm" .. farm .. ".static.flickr.com/".. server .. "/" .. id .. "_" .. secret .. "_m.jpg"
      Rainlendar_Log("Downloading: " .. url)
      Rainlendar_DownloadElement(url, Shadow4_Photos_OnDownloadElement, "bitmap.photos.photo." .. window, skin, window)

      if (ownername and title and page and pages) then
        Rainlendar_SetItemValue(window, "Photos.notes", "text", title)
        Rainlendar_SetItemValue(window, "Photos.author", "text", ownername)
        Rainlendar_SetItemValue(window, "Photos.index", "text", page .. "/" .. pages)
      end
      
      -- Change the action in the show-button so it'll go to correct page
		Rainlendar_SetItemValue(window, "Photos.button.show", "action", "Shadow4_Photos_Execute('http://www.flickr.com/photos/" .. owner .. "/" .. id .. "/')")
    else
      Rainlendar_SetItemValue(window, "Photos.infotext", "text", Rainlendar_GetString("Unknown result!"))
      Rainlendar_SetItemValue(window, "Photos.infotext", "visible", "1")
      Rainlendar_Redraw(0, window)
    end
  else
    -- Download failed
    Rainlendar_SetItemValue(window, "Photos.infotext", "text", Rainlendar_GetString("Download failed!"))
    Rainlendar_Redraw(0, window)
  end
end

--
-- Another download callback which is called after the image has been downloaded
-- as an element. This just redraws the windows so the image appears on the screen.
--
function Shadow4_Photos_OnDownloadElement(result, data, userData)
  Rainlendar_SetItemValue(userData, "Photos.infotext", "visible", "0")
  Rainlendar_SetItemValue(userData, "Photos.photo", "visible", "1")
  Rainlendar_Redraw(0, userData)
end

function Shadow4_Photos_OnMouseEnter(eventData, window, userData)
  Rainlendar_SetItemValue(window, "Photos.button.left", "visible", "1")
  Rainlendar_SetItemValue(window, "Photos.button.show", "visible", "1")
  Rainlendar_SetItemValue(window, "Photos.button.random", "visible", "1")
  Rainlendar_SetItemValue(window, "Photos.button.right", "visible", "1")
  Rainlendar_SetItemValue(window, "Photos.background.dark", "visible", "1")
  Rainlendar_SetItemValue(window, "Photos.notes", "visible", "1")
  Rainlendar_SetItemValue(window, "Photos.author", "visible", "1")
  Rainlendar_SetItemValue(window, "Photos.index", "visible", "1")
  Rainlendar_Redraw(0, window)
end

function Shadow4_Photos_OnMouseLeave(eventData, window, userData)
  Rainlendar_SetItemValue(window, "Photos.button.left", "visible", "0")
  Rainlendar_SetItemValue(window, "Photos.button.show", "visible", "0")
  Rainlendar_SetItemValue(window, "Photos.button.random", "visible", "0")
  Rainlendar_SetItemValue(window, "Photos.button.right", "visible", "0")
  Rainlendar_SetItemValue(window, "Photos.background.dark", "visible", "0")
  Rainlendar_SetItemValue(window, "Photos.notes", "visible", "0")
  Rainlendar_SetItemValue(window, "Photos.author", "visible", "0")
  Rainlendar_SetItemValue(window, "Photos.index", "visible", "0")
  Rainlendar_Redraw(0, window)
end

-- Handler for the left button
function Shadow4_Photos_Left(skin, window)
  _, _, _, key = window:find("(.*)::(.*)")
  
 data = Rainlendar_GetItemValue(window, "Photos.index", "text")
  _, _, page = data:find("(.*)/(.*)")
  if page then
    page = tonumber(page) - 1
    if page >= 1 then
      Shadow4_Photos_Download(page, skin .. "|" .. key)
    end
  end
end

-- Handler for the right button
function Shadow4_Photos_Right(skin, window)
  _, _, _, key = window:find("(.*)::(.*)")

  data = Rainlendar_GetItemValue(window, "Photos.index", "text")
  _, _, page, total = data:find("(.*)/(.*)")
  if page then
    page = tonumber(page) + 1
    if page <= tonumber(total) then
      Shadow4_Photos_Download(page, skin .. "|" .. key)
    end
  end
end

-- Handler for the random button
function Shadow4_Photos_Random(skin, window)
  _, _, _, key = window:find("(.*)::(.*)")

  if #g_Shadow4_Photos_Cache[key] > 0 then
    page = math.random(#g_Shadow4_Photos_Cache[key])
  else
    page = math.random(500)
  end
  Shadow4_Photos_Download(page, skin .. "|" .. key)
end

-- Handler for the show button
function Shadow4_Photos_Execute(url)
  Rainlendar_Execute(url)
end
