-- The list of urls which are parsed. Note that you cannot just add any url here.
-- The contents must be exactly like the parser assumes or it cannot read the data.
-- So at the moment this works only with the elisa.net's tv programs.
-- The {DATE} will be replaced with the current date (YYYYMMDD).
g_Shadow4_TV_Urls = { 
  "http://elisa.net/elisatv/tvjaradio/pda/kanava.tv?k=TV1&pvm={DATE}",
  "http://elisa.net/elisatv/tvjaradio/pda/kanava.tv?k=TV2&pvm={DATE}",
  "http://elisa.net/elisatv/tvjaradio/pda/kanava.tv?k=MTV&pvm={DATE}",
  "http://elisa.net/elisatv/tvjaradio/pda/kanava.tv?k=NEL&pvm={DATE}",
  "http://elisa.net/elisatv/tvjaradio/pda/kanava.tv?k=TVT&pvm={DATE}",
  "http://elisa.net/elisatv/tvjaradio/pda/kanava.tv?k=NEP&pvm={DATE}"
}

-- This is a table which will contain the downloaded program data
g_Shadow4_TV_Cache = {}

-- Stores the current date which is used to check if the programs should be downloaded again
g_Shadow4_TV_Date = 0

--
-- This is the initialization function. It gets called every time Rainlendar
-- is started or refreshed. The initialization function must be defined in the
-- skin file.
--
function Shadow4_TV_Initialize(skin, window)
  Rainlendar_CreateTimer(60 * 1000, Shadow4_TV_OnTimer, skin .. "|" .. window)
  Rainlendar_SetEventHandler(Shadow4_TV_OnPowerResume, 1000, "", skin .. "|" .. window)
  Shadow4_TV_DownloadPrograms(skin .. "|" .. window)
end

function Shadow4_TV_DownloadPrograms(userData)
  _, _, skin, window = userData:find("(.*)|(.*)")

  -- Erase the existing items  
  Rainlendar_SetItemValue(window, "TV.list", "list", "")
  g_Shadow4_TV_Cache = {}  -- Clear the cache

  g_Shadow4_TV_Date = os.date("%Y%m%d")

  pos = 1
    for _, url in ipairs(g_Shadow4_TV_Urls) do
    Rainlendar_SetItemValue(window, "TV.list", "list." .. pos .. ".type", "1")
    Rainlendar_SetItemValue(window, "TV.list", "list." .. pos .. ".text", Rainlendar_GetString("Loading..."))

    -- Replace the {DATE} with a current day's date
    url = string.gsub(url, "{DATE}", g_Shadow4_TV_Date)

        Rainlendar_Download(url, Shadow4_TV_OnDownload, userData .. "|" .. pos)
        pos = pos + 1
    end
    Rainlendar_Redraw(1, window)
end

--
-- A callback function for the timer. This gets called every time the timer triggers.
-- Returns true so the timer will go on forever (false would stop the timer).
--
function Shadow4_TV_OnTimer(userData)
  _, _, skin, window = userData:find("(.*)|(.*)")

  date = os.date("%Y%m%d")

  -- Check if new programs should be downloaded. Note that this doesn't work correctly since the next day's 
  -- programs start from around 4:00 and not from midnight. 
  -- TODO: Download new programs only after the last in the cache has been started
  if (g_Shadow4_TV_Date == date) then
    time = tonumber(os.date("%H")) * 60 + tonumber(os.date("%M"));

    -- TODO: If the day has been changed increase the time by 24 * 60 minutes
  
    -- Check which programs should be shown
    pos = 1
      for _, listing in ipairs(g_Shadow4_TV_Cache) do
        found = 0
        for _, item in ipairs(listing) do
          if (item["start"] > time) then
            break
          end
          found = found + 1
      end
  
        if (found > 0 and found < #listing) then
          item = listing[found]
        Rainlendar_SetItemValue(window, "TV.list", "list." .. pos .. ".text", item["text"])
  
        -- Set the progressbar
        if (found < #listing) then
          nextItem = listing[found + 1]
          duration = nextItem["start"] - item["start"]
          progress = ((time - item["start"]) *  100) / duration
          Rainlendar_SetItemValue(window, "TV.list", "list." .. pos .. ".progress", progress)
        end

        -- Set the tooltip
        tooltip = ""
        for i = found,#listing do
            item = listing[i]
          tooltip = tooltip .. item["text"] .. "\n"
        end
        Rainlendar_SetItemValue(window, "TV.list", "list." .. pos .. ".tooltipHeader", Rainlendar_GetString("Upcoming programs:"))
        Rainlendar_SetItemValue(window, "TV.list", "list." .. pos .. ".tooltipText", tooltip)
      end
      
      pos = pos + 1  
    end
  
      Rainlendar_Redraw(1, window)
  else
    Shadow4_TV_DownloadPrograms(userData)
  end
  
  return True
end

-- A callback function which gets called when computer wakes up from sleep.
-- This updates the TV programs and loads them from the net if necessary.
function Shadow4_TV_OnPowerResume(eventData, userData)
  Shadow4_TV_OnTimer(userData)
end

--
-- A callback function for the downloading. This gets called after the given file
-- has been downloaded from the net. The data is the downloaded file.
-- In this case the TV feed is parsed and the list is filled with the data.
--
function Shadow4_TV_OnDownload(result, data, userData)
  _, _, skin, window, pos = userData:find("(.*)|(.*)|(.*)")

  -- Parse the data and fill in the list item data
  if (result == 200) then
    -- Download succeeded
  
    -- Format is like this:
    --   <b>13.15</b> <a href="http://elisa.net/elisatv/tvjaradio/tv/../pda/ohjelma.tv?id=NEP20080510008">Selviytymisopas</a><br />
    -- <b>02.00</b> Taitopeli.com<br />

    listing = {}

    -- Find all the items
    prev = 0
    for item in string.gmatch(data, "<b>%d%d.%d%d</b>.-<br />") do
      _, _, hour, min = item:find("<b>(%d%d).(%d%d)</b>")
      _, _, link = item:find("<a href=\"(.-)\">")
      _, _, title = item:find("</b> (.-)<br />")
      title = string.gsub(title, "<.->", "")
      
      -- The text in the web page is encoded with iso-8859-1 but Rainlendar expects UTF-8 so we need to convert it
      title = Rainlendar_ConvertString(title, "iso-8859-1")
      
      -- Do some other replacements to the title
      title = string.gsub(title, "&amp;", "&")

      -- Create the item            
      if (hour and min and title) then
        text = hour .. ":" .. min .. " "
        if (link) then
          text = text .. "[[" .. link .. "][" .. title .. "]]"
        else
          text = text .. title
        end
        
        data = {}
        data["start"] = tonumber(hour) * 60 + tonumber(min)
        data["text"] = text
        
        -- Check times which are on the next day
        if (data["start"] < prev) then
          data["start"] = data["start"] + (24 * 60)
        end
        prev = data["start"]
        
        table.insert(listing, data) 
      end
    end

    g_Shadow4_TV_Cache[tonumber(pos)] = listing

    if (#listing == 0) then
        Rainlendar_SetItemValue(window, "TV.list", "list." .. pos .. ".text", Rainlendar_GetString("Parsing failed!"))
    end
  else
    -- Download failed
    Rainlendar_SetItemValue(window, "TV.list", "list." .. pos .. ".text", Rainlendar_GetString("Download failed!"))
  end
  
  Shadow4_TV_OnTimer(skin .. "|" .. window)
end
