--
-- This is the initialization function. It gets called every time Rainlendar
-- is started or refreshed. The initialization function must be defined in the
-- skin file.
--
function Shadow4_YahooWeather_Initialize(skin, window)
  -- Create a timer which updates the data once per hour
  Rainlendar_CreateTimer(60 * 60 * 1000, Shadow4_YahooWeather_OnTimer, skin .. "|" .. window)

  -- Set global event for power resume
  Rainlendar_SetEventHandler(Shadow4_YahooWeather_OnPowerResume, 1000, "", skin .. "|" .. window)
  
  -- Run the timer callback so the data is updated immediately
  Shadow4_YahooWeather_OnTimer(skin .. "|" .. window)
end

--
-- A callback function for the timer. This gets called every time the timer triggers.
-- Returns true so the timer will go on forever (false would stop the timer).
--
function Shadow4_YahooWeather_OnTimer(userData)
  -- Split the user data
  _, _, skin, window = userData:find("(.*)|(.*)")

  woeid = Rainlendar_GetVariable(skin, window, "Weather_woeid")
  celsius = Rainlendar_GetVariable(skin, window, "Weather_celsius")

  if woeid then
    url = "http://weather.yahooapis.com/forecastrss?w=" .. woeid
    if celsius and celsius == "1" then
        url = url .. "&u=c" 
    end
    Rainlendar_Log("Downloading: " .. url)
    Rainlendar_Download(url, Shadow4_YahooWeather_OnDownload, userData)
    return True
  else
    Rainlendar_Log("The woeid variable is not defined.")
    return False
  end
end

-- A callback function which gets called when computer wakes up from sleep.
-- This reloads the weather data.
function Shadow4_YahooWeather_OnPowerResume(eventData, userData)
  Shadow4_YahooWeather_OnTimer(userData)
end

--
-- A callback function for the downloading. This gets called after the given file
-- has been downloaded from the net. The data is the downloaded file.
-- In this case the weather rss feed is parsed and the view is filled
-- in with the data.
--
function Shadow4_YahooWeather_OnDownload(result, data, userData)
  -- Split the user data
  _, _, skin, window = userData:find("(.*)|(.*)")

  iconMap = {}
  iconMap["0"] = "windy" -- tornado
  iconMap["1"] = "windy" -- tropical storm
  iconMap["2"] = "windy" -- hurricane
  iconMap["3"] = "thunderstorm" -- severe thunderstorms
  iconMap["4"] = "thunderstorm" -- thunderstorms
  iconMap["5"] = "rain_snow" -- mixed rain and snow
  iconMap["6"] = "rain_snow" -- mixed rain and sleet
  iconMap["7"] = "snow" -- mixed snow and sleet
  iconMap["8"] = "rain_snow" -- freezing drizzle
  iconMap["9"] = "rain_light" -- drizzle
  iconMap["10"] = "rain_snow" -- freezing rain
  iconMap["11"] = "rain_light" -- showers
  iconMap["12"] = "rain_light" -- showers
  iconMap["13"] = "snow_light" -- snow flurries
  iconMap["14"] = "snow_light" -- light snow showers
  iconMap["15"] = "snow_light" -- blowing snow
  iconMap["16"] = "snow" -- snow
  iconMap["17"] = "snow" -- hail
  iconMap["18"] = "sleet" -- sleet
  iconMap["19"] = "fog" -- dust
  iconMap["20"] = "fog" -- foggy
  iconMap["21"] = "fog" -- haze
  iconMap["22"] = "fog" -- smoky
  iconMap["23"] = "fog" -- blustery
  iconMap["24"] = "windy" -- windy
  iconMap["25"] = "clear_day" -- cold
  iconMap["26"] = "cloudy" -- cloudy
  iconMap["27"] = "cloudy_mostly_night" -- mostly cloudy (night)
  iconMap["28"] = "cloudy_mostly_day" -- mostly cloudy (day)
  iconMap["29"] = "cloudy_partly_night" -- partly cloudy (night)
  iconMap["30"] = "cloudy_partly_day" -- partly cloudy (day)
  iconMap["31"] = "clear_night" -- clear (night)
  iconMap["32"] = "clear_day" -- sunny
  iconMap["33"] = "cloudy_partly_night" -- fair (night)
  iconMap["34"] = "cloudy_partly_day" -- fair (day)
  iconMap["35"] = "rain_snow" -- mixed rain and hail
  iconMap["36"] = "clear_day" -- hot
  iconMap["37"] = "thunderstorm" -- isolated thunderstorms
  iconMap["38"] = "thunderstorm" -- scattered thunderstorms
  iconMap["39"] = "thunderstorm" -- scattered thunderstorms
  iconMap["40"] = "rain_light" -- scattered showers
  iconMap["41"] = "snow" -- heavy snow
  iconMap["42"] = "snow_light" -- scattered snow showers
  iconMap["43"] = "snow" -- heavy snow
  iconMap["44"] = "cloudy_partly_day" -- partly cloudy
  iconMap["45"] = "thunderstorm" -- thundershowers
  iconMap["46"] = "snow_light" -- snow showers
  iconMap["47"] = "thunderstorm" -- isolated thundershowers
  iconMap["3200"] = "na" -- not available

  if (result == 200) then

    -- <link>http://us.rd.yahoo.com/dailynews/rss/weather/Tampere__FI/*http://weather.yahoo.com/forecast/FIXX0031_c.html</link>
    -- <yweather:location city="Tampere" region=""   country="Finland"/>
    -- <yweather:units temperature="C" distance="km" pressure="mb" speed="km/h"/>
    -- <yweather:wind chill="12"   direction="130"   speed="19.31" />
    -- <yweather:atmosphere humidity="94"  visibility="9.99"  pressure="1015.92"  rising="0" />
    -- <yweather:astronomy sunrise="6:12 am"   sunset="8:25 pm"/>
    -- <yweather:condition  text="Mostly Cloudy"  code="28"  temp="12"  date="Sat, 01 Sep 2012 8:50 am EEST" />

    -- Create a link from the title
    _, _, title = data:find("<yweather:location city=\"(.-)\"")
    _, _, link = data:find("<link>(.-)</link>")

    if title and link then
      title = "[[" .. link .. "][" .. title .. "]]" 

      -- Put the title to the window
      Rainlendar_SetItemValue(window, "Weather.title", "text", title)

      -- Get the units
      _, _, temperatureUnit, distanceUnit, pressureUnit, speedUnit = data:find("<yweather:units temperature=\"(.-)\" distance=\"(.-)\" pressure=\"(.-)\" speed=\"(.-)\"/>")

      -- Current conditions
      _, _, conditions, code, degrees, date = data:find("<yweather:condition  text=\"(.-)\"  code=\"(.-)\"  temp=\"(.-)\"  date=\"(.-)\" />")

      if conditions and code and degrees and date and code then
        -- Set the icon
        Rainlendar_SetItemValue(window, "Weather.icon.large", "element", "bitmap.icon.weather." .. iconMap[code])

        -- Set the conditions
        Rainlendar_SetItemValue(window, "Weather.degrees", "text", degrees .. "°" .. temperatureUnit)

        -- <yweather:wind chill="12"   direction="130"   speed="19.31" />
         _, _, chill, direction, speed = data:find("<yweather:wind chill=\"(.-)\"   direction=\"(.-)\"   speed=\"(.-)\" />")

        -- <yweather:atmosphere humidity="94"  visibility="9.99"  pressure="1015.92"  rising="0" />
         _, _, humidity, visibility, pressure, rising = data:find("<yweather:atmosphere humidity=\"(.-)\"  visibility=\"(.-)\"  pressure=\"(.-)\"  rising=\"(.-)\" />")

        -- <yweather:astronomy sunrise="6:12 am"   sunset="8:25 pm"/>
         _, _, sunrise, sunset = data:find("<yweather:astronomy sunrise=\"(.-)\"   sunset=\"(.-)\"/>")
                    
        text = ""
        text = text .. conditions .. " " .. degrees .. "°" .. temperatureUnit .. "\n"
        if chill and direction and speed then
          text = text .. Rainlendar_GetString("Chill:") .. " " .. chill .. "°" .. temperatureUnit .. "\n"
          text = text .. Rainlendar_GetString("Direction:") .. " " .. direction .. "°\n"
          text = text .. Rainlendar_GetString("Speed:") .. " " .. speed .. " " .. speedUnit .. "\n"
        end
        if humidity and visibility and pressure and rising then
          text = text .. Rainlendar_GetString("Humidity:") .. " " .. humidity .. "%\n"
          text = text .. Rainlendar_GetString("Visibility:") .. " " .. visibility .. " " .. distanceUnit .. "\n"
          text = text .. Rainlendar_GetString("Pressure:") .. " " .. pressure .. " " .. pressureUnit
          if rising == "1" then
            text = text .. " (" .. Rainlendar_GetString("raising") .. ")\n"
          elseif rising == "2" then
            text = text .. " (" .. Rainlendar_GetString("falling") .. ")\n"
          else
            text = text .. " (" .. Rainlendar_GetString("steady") .. ")\n"
          end
        end
        if sunrise and sunset then
          text = text .. Rainlendar_GetString("Sunrise:") .. " " .. sunrise .. "\n"
          text = text .. Rainlendar_GetString("Sunset:") .. " " .. sunset .. "\n"
        end

        text = text .. date
        text = text .. "\n\n" .. Rainlendar_GetString("Provider:") .. " [[http://weather.yahoo.com][Yahoo! Weather]]"
    
        Rainlendar_SetItemValue(window, "Weather.icon.large", "tooltipHeader", title)
        Rainlendar_SetItemValue(window, "Weather.icon.large", "tooltipText", text)
    
        -- Forecast
        -- <yweather:forecast day="Sat" date="1 Sep 2012" low="11" high="13" text="Light Rain" code="11" />
    
          -- Loop throught all <yweather:forecast>:s.
          count = 1
          for forecast in string.gmatch(data, "<yweather:forecast(.-)/>") do

Rainlendar_Log(forecast)

            -- <yweather:forecast day="Sun" date="2 Sep 2012" low="12" high="17" text="Showers" code="11" />
            _, _, day = forecast:find("day=\"(.-)\"")
            _, _, low = forecast:find("low=\"(.-)\"")
            _, _, high = forecast:find("high=\"(.-)\"")
            _, _, code = forecast:find("code=\"(.-)\"")

            if day and low and high and code then
              day = Rainlendar_GetString(day)
              day = Shadow4_YahooWeather_Uppercase(day)
              Rainlendar_SetItemValue(window, "Weather.day." .. count, "text", day)
              Rainlendar_SetItemValue(window, "Weather.icon.small." .. count, "element", "bitmap.icon.weather." .. iconMap[code])
              Rainlendar_SetItemValue(window, "Weather.degrees.low." .. count, "text", Rainlendar_GetString("Min:") .. " " .. low .. "°" .. temperatureUnit)
              Rainlendar_SetItemValue(window, "Weather.degrees.high." .. count, "text", Rainlendar_GetString("Max:") .. " " .. high .. "°" .. temperatureUnit)
            end

            count = count + 1
          end
    
      else
        Rainlendar_SetItemValue(window, "Weather.title", "text", Rainlendar_GetString("No data"))
      end

  else
    -- Download failed
    Rainlendar_SetItemValue(window, "Weather.title", "text", Rainlendar_GetString("WOEID error!"))
  end

  else
    -- Download failed
    Rainlendar_SetItemValue(window, "Weather.title", "text", Rainlendar_GetString("Download failed!"))
  end
  
  Rainlendar_Redraw(0, window)
end

-- Converts the first character of a string to uppercase
function Shadow4_YahooWeather_Uppercase(s)
  first = string.sub(s, 1, 1)
  s = string.sub(s, 2)
  first = string.upper(first)
  return first .. s
end

-- Converts the &#NNN; values to utf-8
function Shadow4_YahooWeather_DecodeString(s)
  s = string.gsub(s, "&#(%d+);", 
    function (d)
      val = tonumber(d, 10)
      if (val < 128) then
        return string.char(val)
      elseif (val < 192) then 
        return "\194" .. string.char(val)
      else
        return "\195" .. string.char(val - 64)
      end
    end)
  return s
end
